package org.openxma.dsl.generator.helper.style;

import java.util.Map;

import org.openxma.dsl.generator.helper.PresentationModelExtension;
import org.openxma.dsl.pom.model.StylePropertyBackgroundColor;
import org.openxma.dsl.pom.model.StylePropertyFont;
import org.openxma.dsl.pom.model.StylePropertyForegroundColor;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;

import at.spardat.xma.guidesign.CustomStylesCollection;
import at.spardat.xma.guidesign.GuidesignFactory;
import at.spardat.xma.guidesign.XMAWidget;
import at.spardat.xma.guidesign.types.ColorName;
import at.spardat.xma.guidesign.types.FontStyle;
import at.spardat.xma.guidesign.types.NamedFlag;
import at.spardat.xma.guidesign.types.TypesFactory;
import at.spardat.xma.guidesign.types.XMAColor;
import at.spardat.xma.guidesign.types.XMAFont;

public class ColorAndFontUtil {
    
    /**
     * Set colors and font of a widget according the defined styles
     * 
     * @param styleProperties
     * @param widget
     */
    public static void setColorsAndFont(Map<Integer, StyleSpecificationProperty> styleProperties, XMAWidget widget) {
        StylePropertyForegroundColor stylePropertyForegroundColor = PresentationModelExtension.getStylePropertyForegroundColor(styleProperties);
        if (stylePropertyForegroundColor != null) {
            XMAColor xmaColor = createXMAColor(stylePropertyForegroundColor.getColorID(),stylePropertyForegroundColor.getRed(),stylePropertyForegroundColor.getGreen(),stylePropertyForegroundColor.getBlue());    
            widget.setForegroundColor(xmaColor);
        }
        StylePropertyBackgroundColor stylePropertyBackgroundColor = PresentationModelExtension.getStylePropertyBackgroundColor(styleProperties);
        if (stylePropertyBackgroundColor != null) {
            XMAColor xmaColor = createXMAColor(stylePropertyBackgroundColor.getColorID(),stylePropertyBackgroundColor.getRed(),stylePropertyBackgroundColor.getGreen(),stylePropertyBackgroundColor.getBlue());                
            widget.setBackgroundColor(xmaColor);
        }            
        StylePropertyFont stylePropertyFont = PresentationModelExtension.getStylePropertyFont(styleProperties);
        if (stylePropertyFont != null) {
            XMAFont xmaFont = TypesFactory.eINSTANCE.createXMAFont();
            if (stylePropertyFont.getName().size()>0) {
                xmaFont.setName(stylePropertyFont.getName().get(0));
            }
            if (stylePropertyFont.getSize()>0) {
                xmaFont.setSize(stylePropertyFont.getSize());
            } else {
                xmaFont.setSize(-1);
            }
            if (stylePropertyFont.getStyles().size()>0) {
                // TODO Refactore styles in guidesign model
                // Quit difficult to handle! 
                // Not only difficult but also consumes additional space -> should be a simple Set
                CustomStylesCollection csc = GuidesignFactory.eINSTANCE.createCustomStylesCollection();
                csc.addStyles((FontStyle[]) FontStyle.VALUES.toArray(new FontStyle[0]));                    
                xmaFont.setStyles(csc);                  
                for (String style : stylePropertyFont.getStyles()) {
                    setFontStyle(xmaFont,style);                   
                }                    
            }
            widget.setFont(xmaFont);
        }
    }

    /**
     * Create a XMAColor from the color information of a style
     * 
     * @param colorID
     * @param red
     * @param green
     * @param blue
     * @return
     */
    private static XMAColor createXMAColor(String colorID, int red, int green, int blue) {
        XMAColor xmaColor = TypesFactory.eINSTANCE.createXMAColor();
        if (colorID != null) {
            ColorName colorName = getColorNameForColorId(colorID);
            xmaColor.setColorName(colorName);
            xmaColor.setRed(-1);
            xmaColor.setGreen(-1); 
            xmaColor.setBlue(-1);            
        } else {      
            xmaColor.setRed(red);
            xmaColor.setGreen(green);                    
            xmaColor.setBlue(blue);                    
        }
        return xmaColor;
    }
    
    /**
     * Create a ColorName from a color id
     * 
     * @param colorID
     * @return
     */
    private static ColorName getColorNameForColorId(String colorID) {
        String colorNameForColorId = "COLOR_"+colorID.toUpperCase();
        for (Object obj : ColorName.VALUES) {
            ColorName colorName = (ColorName)obj;
            if (colorName.getName().equalsIgnoreCase(colorNameForColorId)) {
                return colorName;
            }            
        }
        // Color not found
        throw new RuntimeException("Color not supported: '"+colorID+"'");
    }

    /**
     * Set the style of a XMAFont object
     * 
     * @param xmaFont
     * @param fontStyleName
     */
    private static void setFontStyle(XMAFont xmaFont, String fontStyleName) {
        fontStyleName = fontStyleName.toUpperCase();
        if ("NORMAL".equals(fontStyleName)) {
            fontStyleName="STANDARD";
        } 
        for (Object styleObject : xmaFont.getStyles().getStyles()) {
            NamedFlag namedFlag = (NamedFlag)styleObject;
            if (fontStyleName.equals(namedFlag.getName())) {
                namedFlag.setValue(true);
                return;
            }
        }
        // Font style not found
        throw new RuntimeException("Font style not supported: '"+fontStyleName+"'");
    }       

}
